import {EditorQuestion} from "inquirer";
import {InquireFunction} from "../Inquire.js";
import {personaPossessive, PersonaPrompt} from "../../datatypes/Persona.js";
import {EntryMainMenuChoice, makeEntryMainMenuChoice} from "./EntryMainMenuPrompt.js";
import chalk from "chalk";
import pluralize from "pluralize";
import wordcount from "wordcount";
import {asDefault} from "../../utils/Objects.js";

export interface JournalEntryPromptOptions extends Partial<Omit<EditorQuestion, "name" | "type">>, PersonaPrompt {}

export interface JournalEntryPromptDependencies {
    readonly inquire: InquireFunction<EditorQuestion, string>
}

export function journalEntryPrompt(deps: JournalEntryPromptDependencies): {
    (options: JournalEntryPromptOptions): Promise<string|null>,
    mainMenu: EntryMainMenuChoice<"journalEntry">,
} {
    return makeEntryMainMenuChoice({
        property: "journalEntry",
        name: (input) => typeof input === "string" ? `Change journal entry ${chalk.dim(`(currently ${chalk.greenBright(`${pluralize("word", wordcount(input), true)} long`)})`)}` : "Add journal entry",
        key: "j",
        injected: (options) => promptForJournalEntry(options, deps),
        toOptions: asDefault,
        toProperty: (input) => input === null ? undefined : input,
    })
}

export async function promptForJournalEntry(options: JournalEntryPromptOptions, {inquire}: JournalEntryPromptDependencies): Promise<string | null> {
    const {persona} = options
    const result = await inquire({
        type: "editor",
        message: typeof options.default === "string" ? `Edit ${personaPossessive(persona)} journal entry:` : `Type up ${personaPossessive(persona)} journal entry:`,
        ...options,
    })
    const trimmed = result.trimEnd()
    if (trimmed === "") {
        return null
    } else {
        return trimmed
    }
}